@@ -1,6 +1,6 @@
 // file      : xsde/cxx/parser/validating/nmtokens-stl.cxx
 // author    : Boris Kolpackov <boris@codesynthesis.com>
-// copyright : Copyright (c) 2005-2008 Code Synthesis Tools CC
+// copyright : Copyright (c) 2005-2009 Code Synthesis Tools CC
 // license   : GNU GPL v2 + exceptions; see accompanying LICENSE file
 
 #include <xsde/cxx/config.hxx>
@@ -15,10 +15,55 @@ namespace xsde
     {
       namespace validating
       {
+        nmtokens_pimpl::
+        ~nmtokens_pimpl ()
+        {
+          if (!base_)
+            delete seq_;
+        }
+
+        void nmtokens_pimpl::
+        _reset ()
+        {
+          nmtokens_pskel::_reset ();
+
+          if (!base_)
+          {
+            delete seq_;
+            seq_ = 0;
+          }
+
+          parser_._reset ();
+        }
+
+        nmtokens_pimpl::
+        nmtokens_pimpl (bool base)
+            : base_ (base), seq_ (0)
+        {
+        }
+
+        void nmtokens_pimpl::
+        pre_impl (string_sequence* seq)
+        {
+          seq_ = seq;
+        }
+
         void nmtokens_pimpl::
         _pre ()
         {
-          seq_.clear ();
+          if (seq_ == 0)
+          {
+            seq_ = new string_sequence ();
+
+#ifndef XSDE_EXCEPTIONS
+            if (seq_ == 0)
+            {
+              _sys_error (sys_error::no_memory);
+              return;
+            }
+#endif
+          }
+
           nmtokens_pskel::_pre ();
         }
 
@@ -29,15 +74,15 @@ namespace xsde
 
           // Should have at least one element.
           //
-          if (!_context ().error_type () && seq_.size () < 1)
+          if (!_context ().error_type () && seq_->size () < 1)
             _schema_error (schema_error::invalid_nmtokens_value);
         }
 
-        string_sequence nmtokens_pimpl::
+        string_sequence* nmtokens_pimpl::
         post_nmtokens ()
         {
-          string_sequence r;
-          r.swap (seq_);
+          string_sequence* r = seq_;
+          seq_ = 0;
           return r;
         }
 
@@ -68,21 +113,14 @@ namespace xsde
 
 #ifndef XSDE_EXCEPTIONS
             if (!parser_._error_type ())
-              seq_.push_back (x);
+              seq_->push_back (x);
             else
               parser_._copy_error (ctx);
 #else
-            seq_.push_back (x);
+            seq_->push_back (x);
 #endif
           }
         }
-
-        void nmtokens_pimpl::
-        _reset ()
-        {
-          nmtokens_pskel::_reset ();
-          parser_._reset ();
-        }
       }
     }
   }
